@@ -1,4 +1,3 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/apm/apm_driver.c,v 1.65 2003/10/30 17:36:57 tsi Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -19,9 +18,9 @@
 #define APM_VERSION		4000
 #define APM_NAME		"APM"
 #define APM_DRIVER_NAME		"apm"
-#define APM_MAJOR_VERSION       1
-#define APM_MINOR_VERSION       1
-#define APM_PATCHLEVEL          0
+#define APM_MAJOR_VERSION       PACKAGE_VERSION_MAJOR
+#define APM_MINOR_VERSION       PACKAGE_VERSION_MINOR
+#define APM_PATCHLEVEL          PACKAGE_VERSION_PATCHLEVEL
 #ifndef PCI_CHIP_AT3D
 #define PCI_CHIP_AT3D	0x643D
 #endif
@@ -206,8 +205,12 @@ static const char *shadowSymbols[] = {
 
 #ifdef XFree86LOADER
 static const char *miscfbSymbols[] = {
+#ifdef HAVE_XF1BPP
     "xf1bppScreenInit",
+#endif
+#ifdef HAVE_XF4BPP
     "xf4bppScreenInit",
+#endif
     NULL
 };
 #endif
@@ -376,7 +379,7 @@ ApmFindIsaDevice(GDevPtr dev)
 static void
 ApmAssignFPtr(ScrnInfoPtr pScrn)
 {
-    pScrn->driverVersion	= VERSION;
+    pScrn->driverVersion	= APM_VERSION;
     pScrn->driverName		= APM_DRIVER_NAME;
     pScrn->name			= APM_NAME;
     pScrn->Probe		= ApmProbe;
@@ -411,9 +414,11 @@ ApmProbe(DriverPtr drv, int flags)
      * file info to override any contradictions.
      */
 
+#ifndef XSERVER_LIBPCIACCESS
     if (xf86GetPciVideoInfo() == NULL) {
 	return FALSE;
     }
+#endif
     numUsed = xf86MatchPciInstances(APM_NAME, PCI_VENDOR_ALLIANCE,
 		    ApmChipsets, ApmPciChipsets, DevSections, numDevSections,
 		    drv, &usedChips);
@@ -550,12 +555,16 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
     pEnt = pApm->pEnt	= xf86GetEntityInfo(pScrn->entityList[0]);
     if (pEnt->location.type == BUS_PCI) {
 	pApm->PciInfo	= xf86GetPciInfoForEntity(pEnt->index);
+#ifndef XSERVER_LIBPCIACCESS
 	pApm->PciTag	= pciTag(pApm->PciInfo->bus, pApm->PciInfo->device,
 				 pApm->PciInfo->func);
+#endif
     }
     else {
 	pApm->PciInfo	= NULL;
+#ifndef XSERVER_LIBPCIACCESS
 	pApm->PciTag	= 0;
+#endif
     }
 
     if (flags & PROBE_DETECT) {
@@ -653,10 +662,15 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	/* Default to 8 */
 	pScrn->rgbBits = 8;
     }
+#ifndef XSERVER_LIBPCIACCESS
+    /* you're getting a linear framebuffer with pciaccess */
     if (xf86ReturnOptValBool(pApm->Options, OPTION_NOLINEAR, FALSE)) {
 	pApm->noLinear = TRUE;
 	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "No linear framebuffer\n");
     }
+#else
+    pApm->noLinear = FALSE;
+#endif
     from = X_DEFAULT;
     pApm->hwCursor = FALSE;
     if (xf86GetOptValBool(pApm->Options, OPTION_HW_CURSOR, &pApm->hwCursor))
@@ -778,7 +792,7 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
     } else {
 	from = X_PROBED;
 	if (pApm->PciInfo)
-	    pApm->Chipset = pApm->PciInfo->chipType;
+	    pApm->Chipset = PCI_DEV_DEVICE_ID(pApm->PciInfo);
 	else
 	    pApm->Chipset = pEnt->chipset;
 	pScrn->chipset = (char *)xf86TokenToString(ApmChipsets, pApm->Chipset);
@@ -794,7 +808,7 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "ChipRev override: %d\n",
 		   pApm->ChipRev);
     } else if (pApm->PciInfo) {
-	pApm->ChipRev = pApm->PciInfo->chipRev;
+        pApm->ChipRev = PCI_DEV_REVISION(pApm->PciInfo);
     }
 
     /*
@@ -818,7 +832,7 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	pApm->LinAddress = pEnt->device->MemBase;
 	from = X_CONFIG;
     } else if (pApm->PciInfo) {
-	pApm->LinAddress = pApm->PciInfo->memBase[0] & 0xFF800000;
+        pApm->LinAddress = PCI_REGION_BASE(pApm->PciInfo, 0, REGION_MEM) & 0xFF800000;
 	from = X_PROBED;
     } else {
 	/*
@@ -883,9 +897,24 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 	/*unsigned long		save;*/
 	volatile unsigned char	*LinMap;
 
+#ifndef XSERVER_LIBPCIACCESS
 	LinMap = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
 				     pApm->PciTag, pApm->LinAddress,
 				     pApm->LinMapSize);
+#else
+	{
+	    void** result = (void**)&LinMap;
+	    int err = pci_device_map_range(pApm->PciInfo,
+					   pApm->LinAddress,
+					   pApm->LinMapSize,
+					   PCI_DEV_MAP_FLAG_WRITABLE,
+					   result);
+	    
+	    if (err)
+		return FALSE;
+	}
+#endif
+
 	/*save = pciReadLong(pApm->PciTag, PCI_CMD_STAT_REG);
 	pciWriteLong(pApm->PciTag, PCI_CMD_STAT_REG, save | PCI_CMD_MEM_ENABLE);*/
 	d9 = LinMap[0xFFECD9];
@@ -1131,14 +1160,18 @@ ApmPreInit(ScrnInfoPtr pScrn, int flags)
 
     /* Load bpp-specific modules */
     switch (pScrn->bitsPerPixel) {
+#ifdef HAVE_XF1BPP
     case 1:
 	mod = "xf1bpp";
 	req = "xf1bppScreenInit";
 	break;
+#endif
+#ifndef HAVE_XF4BPP
     case 4:
 	mod = "xf4bpp";
 	req = "xf4bppScreenInit";
 	break;
+#endif
     case 8:
     case 16:
     case 24:
@@ -1211,10 +1244,27 @@ ApmMapMem(ScrnInfoPtr pScrn)
     APMDECL(pScrn);
     vgaHWPtr	hwp = VGAHWPTR(pScrn);
 
+#ifndef XSERVER_LIBPCIACCESS
     pApm->LinMap = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
 				 pApm->PciTag,
 				 (unsigned long)pApm->LinAddress,
 				 pApm->LinMapSize);
+#else
+    {
+	void** result = (void**)&pApm->LinMap;
+	int err = pci_device_map_range(pApm->PciInfo,
+				       pApm->LinAddress,
+				       pApm->LinMapSize,
+				       PCI_DEV_MAP_FLAG_WRITABLE |
+				       PCI_DEV_MAP_FLAG_WRITE_COMBINE,
+				       result);
+	
+	if (err) 
+	    return FALSE;
+    }
+#endif
+
+
     if (pApm->LinMap == NULL)
 	return FALSE;
 
@@ -1877,10 +1927,12 @@ ApmScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
     /* Map the chip memory and MMIO areas */
     if (pApm->noLinear) {
-	pApm->saveCmd = pciReadLong(pApm->PciTag, PCI_CMD_STAT_REG);
-	pciWriteLong(pApm->PciTag, PCI_CMD_STAT_REG, pApm->saveCmd | (PCI_CMD_IO_ENABLE|PCI_CMD_MEM_ENABLE));
+	PCI_READ_LONG(pApm->PciInfo, &pApm->saveCmd, PCI_CMD_STAT_REG);
+	PCI_WRITE_LONG(pApm->PciInfo, pApm->saveCmd | (PCI_CMD_IO_ENABLE | PCI_CMD_MEM_ENABLE), PCI_CMD_STAT_REG);
+#ifndef XSERVER_LIBPCIACCESS
 	pApm->FbBase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
 				 pApm->PciTag, 0xA0000, 0x10000);
+#endif
     }
     else
 	if (!ApmMapMem(pScrn))
@@ -1946,18 +1998,22 @@ ApmScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     miSetPixmapDepths();
 
     switch (pScrn->bitsPerPixel) {
+#ifndef HAVE_XF1BPP
     case 1:
 	ret = xf1bppScreenInit(pScreen, FbBase,
 			pScrn->virtualX, pScrn->virtualY,
 			pScrn->xDpi, pScrn->yDpi,
 			pScrn->displayWidth);
 	break;
+#endif
+#ifdef HAVE_XF4BPP
     case 4:
 	ret = xf4bppScreenInit(pScreen, FbBase,
 			pScrn->virtualX, pScrn->virtualY,
 			pScrn->xDpi, pScrn->yDpi,
 			pScrn->displayWidth);
 	break;
+#endif
     case 8:
     case 16:
     case 24:
